events =
{
  -- Called when charging of an entity begins.
  -- entity: the entity that is now charging
  -- charging_entityes: array of player-placed entities associated with charging process.
  on_charging_started = script.generate_event_name(),
  -- Called when an entity is no longer being charged because it moved out of range, something was destroyed, etc.
  -- entity: the affected entity
  -- charging_entityes: array of player-placed entities that were associated with the charging process.
  on_charging_stopped = script.generate_event_name(),
}

remote.add_interface("electric-vehicles-lib", {
  --- Register a new equipment type to be recognized as transformer. It should be a "battery-equipment" type and have its "usage_priority" set to "primary-input".
  -- @param data A table containing these values:
  --   name: the name of the equipment prototype
  ["register-transformer"] = register_transformer,
  --- Register a new equipment type to be recognized as regenerative brake. It should be a "battery-equipment" type and have its "usage_priority" set to "primary-output".
  -- @param data A table containing these values:
  --   name: the name of the equipment prototype
  --   efficiency: the efficiency of energy recovery in range [0,1]
  ["register-brake"] = register_brake,
  --- Specify properties of vehicle prototypes. Default values for the built-in prototypes "car", "tank", "diesel-locomotive" and "cargo-wagon" are automatically set but can be overwritten.
  -- @param data A table containing these values:
  --   name: the name of the entity prototype, must be of type "car", "locomotive" or "cargo-wagon"
  --   mass: the mass in kilograms
  ["set-vehicle-data"] = set_vehicle_data,
  --- Get the registered data of a vehicle type.
  -- @param name The name of the entity prototype.
  -- @return the same table as delivered by "set-entity-data" or nil if no data present
  ["get-vehicle-data"] = get_vehicle_data,
})
